"{ Package: 'stx:libbasic3' }"

Object subclass:#SimulatedClassPool
	instanceVariableNames:'class'
	classVariableNames:''
	poolDictionaries:''
	category:'Kernel-Support'
!

Association subclass:#SimulatedVariableBinding
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:SimulatedClassPool
!

!SimulatedClassPool class methodsFor:'documentation'!

documentation
"
    in contrast to other smalltalks, ST/X does not keep the classVariables
    in a dictionary; instead, classVariables are stored as special globals,
    with the className as prefix (colon-separated).
    The reason is that stc-compiled code should be allowed to access classVars
    in a similar fashion to globals.

    Whenever a classes classPool is requested, an instance of myself is
    created, which forwards at: and at:put: messages to the original class.

    This is an additional goody class; therefore:

    THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTOR ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTOR BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.

    [author:]
        Claus Gittinger
"

! !

!SimulatedClassPool methodsFor:'accessing'!

associationAt:aName
    "simulate an association"

    ^ (aName -> (class classVarAt:aName)) changeClassTo:SimulatedVariableBinding

    "Modified: / 4.2.2000 / 00:27:40 / cg"
!

associationAt:aName ifAbsent:exceptionBlock
    "simulate an association"

    |internalName cls|

    cls := class theNonMetaclass. 
    internalName := (cls name , ':' , aName) asSymbol.
    ^ (internalName -> (cls classVarAt:aName)) changeClassTo:SimulatedVariableBinding

    "Created: / 3.2.2000 / 23:30:07 / cg"
    "Modified: / 4.2.2000 / 01:58:56 / cg"
!

associationsDo:aBlock
    "evaluate aBlock for all of my simulated classVarName->value associations"

    class classVarNames do:[:eachName |
        aBlock value:(self associationAt:eachName)
    ].
!

at:aKey
    ^ class classVarAt:aKey
!

at:aKey ifAbsent:exceptionBlock
    ^ class classVarAt:aKey ifAbsent:exceptionBlock
!

at:aKey put:something
    ^ class classVarAt:aKey put:something
!

keys
    ^ class classVarNames collect:[:nm | nm asSymbol]

    "
     Button classPool keys 
     Button classPool at:#ReturnForm
    "

! !

!SimulatedClassPool methodsFor:'accessing-private'!

setClass:aClass
    class := aClass
! !

!SimulatedClassPool::SimulatedVariableBinding methodsFor:'queries'!

isVariableBinding
     ^ true

    "Created: / 4.2.2000 / 00:27:20 / cg"
! !

!SimulatedClassPool class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/Attic/SimulatedClassPool.st,v 1.4 2000-11-15 12:49:28 cg Exp $'
! !
